//==================================================
//		st7735
//		Library for working with the display with the st7735 driver.
//		Библиотека для работы с дисплеем с драйвером st7735.
//
//  	Version: 1.0.0
//   	Author: Timur Melnikov
//==================================================

#include "stm32f4xx_hal.h"
#include "st7735.h"

static void CS_Reset() {
    HAL_GPIO_WritePin(ST7735_CS_GPIO_Port, ST7735_CS_Pin, GPIO_PIN_RESET);
}

void CS_Set() {
    HAL_GPIO_WritePin(ST7735_CS_GPIO_Port, ST7735_CS_Pin, GPIO_PIN_SET);
}

static void display_Reset() {
    HAL_GPIO_WritePin(ST7735_RES_GPIO_Port, ST7735_RES_Pin, GPIO_PIN_RESET);
    HAL_Delay(5);
    HAL_GPIO_WritePin(ST7735_RES_GPIO_Port, ST7735_RES_Pin, GPIO_PIN_SET);
}

static void display_WriteCommand(uint8_t cmd) {
    HAL_GPIO_WritePin(ST7735_DC_GPIO_Port, ST7735_DC_Pin, GPIO_PIN_RESET);
    HAL_SPI_Transmit(&ST7735_SPI_PORT, &cmd, sizeof(cmd), HAL_MAX_DELAY);
}

static void display_WriteData(uint8_t* buff, size_t buff_size) {
    HAL_GPIO_WritePin(ST7735_DC_GPIO_Port, ST7735_DC_Pin, GPIO_PIN_SET);
    HAL_SPI_Transmit(&ST7735_SPI_PORT, buff, buff_size, HAL_MAX_DELAY);
}

void ST7735_Init() {
	CS_Set()
    display_Reset();
    display_WriteCommand(ST7735_SWRESET);							//Software reset
    HAL_Delay(150);
    display_WriteCommand(ST7735_SLPOUT);							//Out of sleep mode
    HAL_Delay(500);
    display_WriteCommand(ST7735_FRMCTR1);							//Frame rate ctrl - normal mode
    display_WriteData(0x01, sizeof(0x01));
    display_WriteData(0x2C, sizeof(0x2C));
    display_WriteData(0x2D, sizeof(0x2D));
    display_WriteCommand(ST7735_FRMCTR2);							//Frame rate ctrl - idle mode
    display_WriteData(0x01, sizeof(0x01));
    display_WriteData(0x2C, sizeof(0x2C));
    display_WriteData(0x2D, sizeof(0x2D));
    display_WriteCommand(ST7735_FRMCTR2);							//Frame rate ctrl - idle mode
    display_WriteData(0x01, sizeof(0x01));
    display_WriteData(0x2C, sizeof(0x2C));
    display_WriteData(0x2D, sizeof(0x2D));
    display_WriteData(0x01, sizeof(0x01));
    display_WriteData(0x2C, sizeof(0x2C));
    display_WriteData(0x2D, sizeof(0x2D));
    display_WriteCommand(ST7735_INVCTR);							//Invertion off
    display_WriteData(0x07, sizeof(0x07));
    display_WriteCommand(ST7735_PWCTR1);							//Power control1
    display_WriteData(0xA2, sizeof(0xA2));
    display_WriteData(0x02, sizeof(0x02));
    display_WriteData(0x84, sizeof(0x84));
    display_WriteCommand(ST7735_PWCTR2);							//Power control2
    display_WriteData(0xC5, sizeof(0xC5));
    display_WriteCommand(ST7735_PWCTR3);							//Power control3
    display_WriteData(0x0A, sizeof(0x0A));
    display_WriteData(0x00, sizeof(0x00));
    display_WriteCommand(ST7735_PWCTR4);							//Power control4
    display_WriteData(0x8A, sizeof(0x8A));
    display_WriteData(0x2A, sizeof(0x2A));
    display_WriteCommand(ST7735_PWCTR5);							//Power control5
    display_WriteData(0x8A, sizeof(0x8A));
    display_WriteData(0xEE, sizeof(0xEE));
    display_WriteCommand(ST7735_PWCTR3);							//invert off
    display_WriteCommand(ST7735_MADCTL);							//Power control5
    display_WriteData(ST7735_ROTATION, sizeof(ST7735_ROTATION));
    display_WriteCommand(ST7735_COLMOD);							//Color mode
    display_WriteData(0x05, sizeof(0x05));
    display_WriteCommand(ST7735_CASET);								//Column addr set
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x7F, sizeof(0x7F));
    display_WriteCommand(ST7735_RASET);								//Row addr set
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x00, sizeof(0x00));
    display_WriteData(0x7F, sizeof(0x7F));
    display_WriteCommand(ST7735_GMCTRP1);							//Magical unicorn dust
    display_WriteData(0x00, sizeof(0x02));							//DELETE
    display_WriteData(0x00, sizeof(0x1c));							//DELETE
    display_WriteData(0x00, sizeof(0x07));							//DELETE
    display_WriteData(0x7F, sizeof(0x12));							//DELETE
    display_WriteData(0x00, sizeof(0x37));							//DELETE
    display_WriteData(0x00, sizeof(0x32));							//DELETE
    display_WriteData(0x00, sizeof(0x29));							//DELETE
    display_WriteData(0x7F, sizeof(0x2d));							//DELETE
    display_WriteData(0x00, sizeof(0x29));							//DELETE
    display_WriteData(0x00, sizeof(0x25));							//DELETE
    display_WriteData(0x00, sizeof(0x2B));							//DELETE
    display_WriteData(0x7F, sizeof(0x39));							//DELETE
    display_WriteData(0x00, sizeof(0x00));							//DELETE
    display_WriteData(0x00, sizeof(0x01));							//DELETE
    display_WriteData(0x00, sizeof(0x03));							//DELETE
    display_WriteData(0x7F, sizeof(0x10));							//DELETE
    display_WriteCommand(ST7735_GMCTRP1);							//Magical unicorn dust
    display_WriteData(0x00, sizeof(0x03));							//DELETE
    display_WriteData(0x00, sizeof(0x1d));							//DELETE
    display_WriteData(0x00, sizeof(0x07));							//DELETE
    display_WriteData(0x7F, sizeof(0x06));							//DELETE
    display_WriteData(0x00, sizeof(0x2E));							//DELETE
    display_WriteData(0x00, sizeof(0x2C));							//DELETE
    display_WriteData(0x00, sizeof(0x29));							//DELETE
    display_WriteData(0x7F, sizeof(0x2D));							//DELETE
    display_WriteData(0x00, sizeof(0x2E));							//DELETE
    display_WriteData(0x00, sizeof(0x2E));							//DELETE
    display_WriteData(0x00, sizeof(0x37));							//DELETE
    display_WriteData(0x7F, sizeof(0x3F));							//DELETE
    display_WriteData(0x00, sizeof(0x00));							//DELETE
    display_WriteData(0x00, sizeof(0x00));							//DELETE
    display_WriteData(0x00, sizeof(0x02));							//DELETE
    display_WriteData(0x7F, sizeof(0x10));							//DELETE
    display_WriteCommand(ST7735_NORON);								//Normal display on
    HAL_Delay(10);
    display_WriteCommand(ST7735_DISPON);							//Main screen turn on
    HAL_Delay(100);
    CS_Reset();
}

static void display_SetAddressWindow(uint8_t x0, uint8_t y0, uint8_t x1, uint8_t y1) {
    // column address set
    display_WriteCommand(ST7735_CASET);
    uint8_t data[] = { 0x00, x0 + ST7735_XSTART, 0x00, x1 + ST7735_XSTART };
    display_WriteData(data, sizeof(data));

    // row address set
    display_WriteCommand(ST7735_RASET);
    data[1] = y0 + ST7735_YSTART;
    data[3] = y1 + ST7735_YSTART;
    display_WriteData(data, sizeof(data));

    // write to RAM
    display_WriteCommand(ST7735_RAMWR);
}



void ST7735_DrawPixel(uint16_t x, uint16_t y, uint16_t color) {
    if((x >= ST7735_WIDTH) || (y >= ST7735_HEIGHT))
        return;

    CS_Set();

    display_SetAddressWindow(x, y, x+1, y+1);
    uint8_t data[] = { color >> 8, color & 0xFF };
    display_WriteData(data, sizeof(data));

    CS_Reset();
}

void ST7735_FillRectangle(uint16_t x, uint16_t y, uint16_t w, uint16_t h, uint16_t color) {
    // clipping
    if((x >= ST7735_WIDTH) || (y >= ST7735_HEIGHT)) return;
    if((x + w - 1) >= ST7735_WIDTH) w = ST7735_WIDTH - x;
    if((y + h - 1) >= ST7735_HEIGHT) h = ST7735_HEIGHT - y;

    CS_Set();
    display_SetAddressWindow(x, y, x+w-1, y+h-1);

    uint8_t data[] = { color >> 8, color & 0xFF };
    HAL_GPIO_WritePin(ST7735_DC_GPIO_Port, ST7735_DC_Pin, GPIO_PIN_SET);
    for(y = h; y > 0; y--) {
        for(x = w; x > 0; x--) {
            HAL_SPI_Transmit(&ST7735_SPI_PORT, data, sizeof(data), HAL_MAX_DELAY);
        }
    }

    CS_Reset();
}

void ST7735_FillScreen(uint16_t color) {
    ST7735_FillRectangle(0, 0, ST7735_WIDTH, ST7735_HEIGHT, color);
}
